@@ -25,7 +25,6 @@ import org.apache.commons.lang3.StringUtils;
 import org.apache.doris.spark.exception.IllegalArgumentException;
 import org.apache.doris.spark.util.ErrorMessages;
 import org.apache.doris.spark.util.IOUtils;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -48,11 +47,7 @@ public abstract class Settings {
         return value;
     }
 
-    public Integer getIntegerProperty(String name) {
-        return getIntegerProperty(name, null);
-    }
-
-    public Integer getIntegerProperty(String name, Integer defaultValue) {
+    public int getIntegerProperty(String name, int defaultValue) {
         try {
             if (getProperty(name) != null) {
                  return Integer.parseInt(getProperty(name));
@@ -63,17 +58,6 @@ public abstract class Settings {
         return defaultValue;
     }
 
-    public Boolean getBooleanProperty(String name) {
-        return getBooleanProperty(name, null);
-    }
-
-    public Boolean getBooleanProperty(String name, Boolean defaultValue) {
-        if (getProperty(name) != null) {
-            return Boolean.valueOf(getProperty(name));
-        }
-        return defaultValue;
-    }
-
     public Settings merge(Properties properties) {
         if (properties == null || properties.isEmpty()) {
             return this;
