@@ -17,12 +17,9 @@
 
 package org.apache.doris.spark.sql
 
-import org.apache.commons.lang3.exception.ExceptionUtils
+import org.apache.doris.spark.DorisStreamLoad
 import org.apache.doris.spark.cfg.{ConfigurationOptions, SparkSettings}
-import org.apache.doris.spark.exception.DorisException
-import org.apache.doris.spark.jdbc.JdbcUtils
 import org.apache.doris.spark.sql.DorisSourceProvider.SHORT_NAME
-import org.apache.doris.spark.writer.DorisWriter
 import org.apache.spark.SparkConf
 import org.apache.spark.sql.execution.streaming.Sink
 import org.apache.spark.sql.sources._
@@ -30,11 +27,13 @@ import org.apache.spark.sql.streaming.OutputMode
 import org.apache.spark.sql.types.StructType
 import org.apache.spark.sql.{DataFrame, SQLContext, SaveMode}
 import org.slf4j.{Logger, LoggerFactory}
+import java.io.IOException
+import java.util
+
+import org.apache.doris.spark.rest.RestService
 
-import java.util.Properties
 import scala.collection.JavaConverters.mapAsJavaMapConverter
 import scala.util.control.Breaks
-import scala.util.{Failure, Success, Try}
 
 private[sql] class DorisSourceProvider extends DataSourceRegister
   with RelationProvider
@@ -60,19 +59,69 @@ private[sql] class DorisSourceProvider extends DataSourceRegister
 
     val sparkSettings = new SparkSettings(sqlContext.sparkContext.getConf)
     sparkSettings.merge(Utils.params(parameters, logger).asJava)
-
-    mode match {
-      case SaveMode.Overwrite =>
-        truncateTable(sparkSettings)
-      case _: SaveMode => // do nothing
-    }
-
-    // accumulator for transaction handling
-    val acc = sqlContext.sparkContext.collectionAccumulator[Long]("BatchTxnAcc")
     // init stream loader
-    val writer = new DorisWriter(sparkSettings, acc)
-    writer.write(data)
+    val dorisStreamLoader = new DorisStreamLoad(sparkSettings, data.columns)
 
+    val maxRowCount = sparkSettings.getIntegerProperty(ConfigurationOptions.DORIS_SINK_BATCH_SIZE, ConfigurationOptions.SINK_BATCH_SIZE_DEFAULT)
+    val maxRetryTimes = sparkSettings.getIntegerProperty(ConfigurationOptions.DORIS_SINK_MAX_RETRIES, ConfigurationOptions.SINK_MAX_RETRIES_DEFAULT)
+
+    data.rdd.foreachPartition(partition => {
+      val rowsBuffer: util.List[util.List[Object]] = new util.ArrayList[util.List[Object]](maxRowCount)
+      partition.foreach(row => {
+        val line: util.List[Object] = new util.ArrayList[Object]()
+        for (i <- 0 until row.size) {
+          val field = row.get(i)
+          line.add(field.asInstanceOf[AnyRef])
+        }
+        rowsBuffer.add(line)
+        if (rowsBuffer.size > maxRowCount) {
+          flush
+        }
+      })
+      // flush buffer
+      if (!rowsBuffer.isEmpty) {
+        flush
+      }
+
+      /**
+       * flush data to Doris and do retry when flush error
+       *
+       */
+      def flush = {
+        val loop = new Breaks
+        loop.breakable {
+
+          for (i <- 1 to maxRetryTimes) {
+            try {
+              dorisStreamLoader.loadV2(rowsBuffer)
+              rowsBuffer.clear()
+              loop.break()
+            }
+            catch {
+              case e: Exception =>
+                try {
+                  logger.debug("Failed to load data on BE: {} node ", dorisStreamLoader.getLoadUrlStr)
+                  //If the current BE node fails to execute Stream Load, randomly switch to other BE nodes and try again
+                  dorisStreamLoader.setHostPort(RestService.randomBackendV2(sparkSettings, logger))
+                  Thread.sleep(1000 * i)
+                } catch {
+                  case ex: InterruptedException =>
+                    logger.warn("Data that failed to load : " + dorisStreamLoader.listToString(rowsBuffer))
+                    Thread.currentThread.interrupt()
+                    throw new IOException("unable to flush; interrupted while doing another attempt", e)
+                }
+            }
+          }
+
+          if (!rowsBuffer.isEmpty) {
+            logger.warn("Data that failed to load : " + dorisStreamLoader.listToString(rowsBuffer))
+            throw new IOException(s"Failed to load data on BE: ${dorisStreamLoader.getLoadUrlStr} node and exceeded the max retry times.")
+          }
+        }
+
+      }
+
+    })
     new BaseRelation {
       override def sqlContext: SQLContext = unsupportedException
 
@@ -94,50 +143,6 @@ private[sql] class DorisSourceProvider extends DataSourceRegister
     sparkSettings.merge(Utils.params(parameters, logger).asJava)
     new DorisStreamLoadSink(sqlContext, sparkSettings)
   }
-
-  private def truncateTable(sparkSettings: SparkSettings): Unit = {
-
-    val feNodes = sparkSettings.getProperty(ConfigurationOptions.DORIS_FENODES)
-    val port = sparkSettings.getIntegerProperty(ConfigurationOptions.DORIS_QUERY_PORT)
-    require(feNodes != null && feNodes.nonEmpty, "doris.fenodes cannot be null or empty")
-    require(port != null, "doris.query.port cannot be null")
-    val feNodesArr = feNodes.split(",")
-    val breaks = new Breaks
-
-    var success = false
-    var exOption: Option[Exception] = None
-
-    breaks.breakable {
-      feNodesArr.foreach(feNode => {
-        Try {
-          val host = feNode.split(":")(0)
-          val url = JdbcUtils.getJdbcUrl(host, port)
-          val props = new Properties()
-          props.setProperty("user", sparkSettings.getProperty(ConfigurationOptions.DORIS_REQUEST_AUTH_USER))
-          props.setProperty("password", sparkSettings.getProperty(ConfigurationOptions.DORIS_REQUEST_AUTH_PASSWORD))
-          val conn = JdbcUtils.getConnection(url, props)
-          val statement = conn.createStatement()
-          val tableIdentifier = sparkSettings.getProperty(ConfigurationOptions.DORIS_TABLE_IDENTIFIER)
-          val query = JdbcUtils.getTruncateQuery(tableIdentifier)
-          statement.execute(query)
-          success = true
-          logger.info(s"truncate table $tableIdentifier success")
-        } match {
-          case Success(_) => breaks.break()
-          case Failure(e: Exception) =>
-            exOption = Some(e)
-            logger.warn(s"truncate table failed on $feNode, error: {}", ExceptionUtils.getStackTrace(e))
-        }
-      })
-
-    }
-
-    if (!success) {
-      throw new DorisException("truncate table failed", exOption.get)
-    }
-
-  }
-
 }
 
 object DorisSourceProvider {
